import cgi
import datetime
import difflib
import glob
import os
import re
import sys
import lxml.etree as etree

TABLES = False
class Doc:
    docs = {}
    def __init__(self, doc_id, doc_version, doc_type):
        self.doc_id = doc_id
        self.doc_version = doc_version
        self.doc_type = doc_type
    @classmethod
    def load_docs(cls):
        for row in eval(open("pub_docs.txt").read()):
            doc = cls(*list(row)[:3])
            cls.docs["CDR%010d.xml" % doc.doc_id] = doc

def sort_us_brand_names(doc):
    for wrapper in doc.findall("DrugInfoMetaData/USBrandNames"):
        names = [name.text for name in wrapper.findall("USBrandName")]
        etree.strip_elements(wrapper, "USBrandName")
        for name in sorted(names):
            etree.SubElement(wrapper, "USBrandName").text = name

def sort_family_cancer_syndromes(doc):
    for genetic_services in doc.findall("GENETICSERVICES"):
        syndromes = {}
        for syndrome in genetic_services.findall("FAMILYCANCERSYNDROME"):
            syndrome_key = [n.text for n in syndrome.findall("SYNDROMENAME")]
            cancer_types = {}
            for node in syndrome.findall("CANCERTYPE"):
                type_name = node.find("TYPENAME").text
                sites = [child.text for child in node.findall("CANCERSITE")]
                key = "%s|%s" % (type_name, "|".join(sites))
                if key not in cancer_types:
                    cancer_types[key] = []
                cancer_types[key].append(node)
            etree.strip_elements(syndrome, "CANCERTYPE")
            for key in sorted(cancer_types):
                syndrome_key.append(key)
                for node in cancer_types[key]:
                    syndrome.append(node)
            syndrome_key = tuple(syndrome_key)
            if syndrome_key not in syndromes:
                syndromes[syndrome_key] = []
            syndromes[syndrome_key].append(syndrome)
        etree.strip_elements(genetic_services, "FAMILYCANCERSYNDROME")
        for key in sorted(syndromes):
            for node in syndromes[key]:
                genetic_services.append(node)

def normalize(xml):
    doc = etree.fromstring(xml)
    sort_family_cancer_syndromes(doc)
    sort_us_brand_names(doc)
    xml = str(etree.tostring(doc, pretty_print=True, encoding="utf-8"), "utf-8")
    wrappers = ("CTGovDisclaimer", "BriefSummary", "DetailedDescription")
    for name in wrappers:
        xml = re.sub("<%s>\\s+" % name, "<%s>" % name, xml)
    xml = re.sub("</Para>\\s*", "</Para>\n", xml)
    return [(line + "\n") for line in xml.splitlines() if line.strip()]

def diff_xml(old, new):
    old, new = normalize(old), normalize(new)
    #open("old-new", "w").write(repr((old, new)))
    if TABLES:
        differ = difflib.HtmlDiff()
        return differ.make_table(old, new, context=True, numlines=1)
    else:
        diffs = [cgi.escape(line) for line in difflib.context_diff(old, new)]
        if not diffs:
            return ""
        return "<pre>" + "".join(diffs) + "</pre>"
        differ = difflib.differ()
        return differ.make_table(old, new, context=True, numlines=1)

docs_with_diffs = 0
Doc.load_docs()
html = ["<!DOCTYPE html>\n<html><head><meta charset='utf-8'><body>"]
for path in glob.glob("new-pub/*.xml"):
    name = os.path.basename(path)
    new = open(path, "rb").read()
    old = open("c:/tmp/old-pub/%s" % name, "rb").read()
    table = diff_xml(old, new)
    if table and "No Differences Found" not in table:
        doc = Doc.docs[name]
        html.append("<h2>%s version %s (%s)</h2>" % (name, doc.doc_version,
                                                     doc.doc_type))
        html.append(table)
        docs_with_diffs += 1
        if docs_with_diffs > 10:
            break
html.append("</body></html>")
html = bytes("\n".join(html) + "\n", "utf-8")
sys.stdout.buffer.write(html)

"""
    # Compare
    diffObj = difflib.Differ()
    diffSeq = diffObj.compare(doc1.splitlines(1),doc2.splitlines(1))

    # If caller only wants changed lines, drop all lines with leading space
    if chgOnly:
        chgSeq = []
        for line in diffSeq:
            if line[0] != ' ':
                chgSeq.append (line)
        # Return them as a (possibly empty) string
        diffText = "".join(chgSeq)

    # Else return entire document as a string
    else:
        diffText = "".join(diffSeq)

    # Convert output back to utf-8.  toprettyxml made it unicode
    if type(diffText) == type(u''):
        diffText = diffText.encode('utf-8')

    return diffText
"""
