import argparse
import datetime
import logging
import socket
import struct
import threading
import lxml.etree as etree

def send_command(command):
    command_set = etree.Element("CdrCommandSet")
    etree.SubElement(command_set, "SessionId").text = Request.session
    wrapper = etree.SubElement(command_set, "CdrCommand")
    wrapper.append(command)
    request = etree.tostring(command_set, encoding="utf-8")
    length = len(request)
    length_bytes = struct.pack("!L", length)
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect(("127.0.0.1", 2019))
    try:
        sock.send(length_bytes)
        sock.sendall(request)
        length_bytes = sock.recv(4)
        if len(length_bytes) != 4:
            raise Exception("server did not send response length")
        length, = struct.unpack("!L", length_bytes)
        if not length:
            raise Exception("server did not send response")
        received = 0
        chunks = []
        while received < length:
            response_bytes = sock.recv(8192)
            if not response_bytes:
                break
            chunks.append(response_bytes)
            received += len(response_bytes)
        if received < length:
            raise Exception("Expected {} bytes reading server response; "
                            "got {} bytes".format(length, received))
        return b"".join(chunks)
    finally:
        sock.close()

class Request:
    responses = []
    lock = threading.Lock()
    def __init__(self):
        attrs = {
            "DocType": "Person",
            "ValidationTypes": "Schema",
            "ErrorLocators": "Y"
        }
        self.command = etree.Element("CdrValidateDoc", **attrs)
        doc_id = etree.SubElement(self.command, "DocId", ValidateOnly="Y")
        doc_id.text = "CDR0000005000"
    def submit(self):
        response = send_command(self.command)
        with Request.lock:
            Request.responses.append(response)

class Thread(threading.Thread):
    def run(self):
        while True:
            with Request.lock:
                if Request.to_do < 1:
                    break
                Request.to_do -= 1
            Request().submit()

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--num-requests", type=int, default=100)
    parser.add_argument("--num-threads", type=int, default=50)
    parser.add_argument("--session", default="guest")
    args = parser.parse_args()
    Request.session = args.session
    Request.to_do = args.num_requests
    start = datetime.datetime.now()
    level = args.num_requests > 1 and logging.INFO or logging.DEBUG
    logging.basicConfig(level=logging.DEBUG)
    threads = []
    for _ in range(args.num_threads):
        threads.append(Thread())
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    elapsed = (datetime.datetime.now() - start).total_seconds()
    logging.info("total processing time: %s", elapsed)
    logging.info("total number of requests: %s", args.num_requests)
    logging.info("seconds per request: %s", elapsed / args.num_requests)
    logging.info("requests per second: %s", args.num_requests / elapsed)
    open("new-val-test.xml", "wb").write(Request.responses[0])
    prev = None
    for response in Request.responses:
        this = etree.XML(response).find("CdrResponse/CdrValidateDocResp")
        this = etree.tostring(this)
        if prev and prev != this:
            raise Exception("%s doesn't match %s" % (prev, this))
        prev = this
main()
