"""
44208 rows fetched
started filtering at 2017-03-25 23:17:39.441000
7520.546 CTGovProtocol
617.616 DrugInformationSummary
6873.616 GlossaryTermName
11427.137 Organization
1592.463 Person
99.983 PoliticalSubUnit
96.969 PoliticalSubUnit
"""

import argparse
import datetime
import lxml.etree as etree
import cdrdb2 as cdrdb
import cdr

class Doc:
    aliases = {
        "Person": "GeneticsProfessional",
        "GlossaryTermName": "GlossaryTerm",
        "DrugInformationSummary": "DrugInfoSummary"
    }
    def __cmp__(self, other):
        return cmp(self.doc_id, other.doc_id)
    def __init__(self, doc_id, doc_version, doc_type, first_pub):
        self.doc_id = doc_id
        self.doc_version = doc_version
        self.doc_type = doc_type
        self.first_pub = first_pub
    def get_vendor_filter_set(self):
        return "Vendor %s Set" % self.aliases.get(self.doc_type, self.doc_type)
    def run_filter_set(self):
        cdr_id = "CDR%010d" % self.doc_id
        filter_set = self.get_vendor_filter_set()
        parms = []
        if self.first_pub:
            parms.append(("DateFirstPub", self.first_pub))
        response = cdr.filterDoc(self.session, ["set:%s" % filter_set], cdr_id,
                                 parm=parms, docVer=self.doc_version,
                                 filterVer="last")
        if type(response) not in (tuple, list):
            print response
            return
        doc, messages = response
        if doc:
            fp = open("old-pub/CDR%010d.xml" % self.doc_id, "wb")
            fp.write(doc)
            fp.close()
        else:
            print(messages)
            #raise Exception("failure filtering CDR%d" % self.doc_id)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--per-doctype", type=int, default=1)
    parser.add_argument("--session")
    args = parser.parse_args()
    Doc.session = args.session
    rows = eval(open("pub_docs.txt").read())
    print("%d rows fetched" % len(rows))
    doc_types = {}
    for doc_id, doc_version, doc_type, first_pub, first_pub_knowable in rows:
        if doc_type not in doc_types:
            doc_types[doc_type] = []
        if first_pub_knowable != "Y":
            first_pub = None
        else:
            first_pub = first_pub and str(first_pub)[:10] or None
        doc = Doc(doc_id, doc_version, doc_type, first_pub)
        doc_types[doc_type].append(doc)
    #print(str(send_command(etree.Element("CdrCacheing", on="term"))))
    cdr.cacheInit(args.session, True, "term")
    start = datetime.datetime.now()
    print("started filtering at %s" % start)
    for doc_type in sorted(doc_types):
        if doc_type in ("CTGovProtocol", "DrugInformationSummary",
                        "GlossaryTermName", "Organization", "Person",
                        "PoliticalSubUnit"):
            continue
        substart = datetime.datetime.now()
        for doc in doc_types[doc_type][:args.per_doctype]:
            doc.run_filter_set()
        print("%s %s" % ((datetime.datetime.now() - substart).total_seconds(),
                         doc_type))
    print("%s TOTAL" % (datetime.datetime.now() - start).total_seconds())
    cdr.cacheInit(args.session, False, "term")
    #print(str(send_command(etree.Element("CdrCacheing", off="term"))))
main()
