import argparse
import datetime
import logging
import cdr
import threading
import lxml.etree as etree

class Request:
    responses = []
    lock = threading.Lock()
    def __init__(self):
        attrs = {
            "DocType": "Person",
            "ValidationTypes": "Schema",
            "ErrorLocators": "Y"
        }
        self.command = etree.Element("CdrValidateDoc", **attrs)
        doc_id = etree.SubElement(self.command, "DocId", ValidateOnly="Y")
        doc_id.text = "CDR0000005000"
    def submit(self):
        response = cdr.valDoc(self.session, "Person", 5000, valLinks="N",
                              errorLocators="Y")
        with Request.lock:
            Request.responses.append(response)

class Thread(threading.Thread):
    def run(self):
        while True:
            with Request.lock:
                if Request.to_do < 1:
                    break
                Request.to_do -= 1
            Request().submit()

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--num-requests", type=int, default=100)
    parser.add_argument("--num-threads", type=int, default=50)
    parser.add_argument("--session", default="guest")
    args = parser.parse_args()
    Request.session = args.session
    Request.to_do = args.num_requests
    start = datetime.datetime.now()
    level = args.num_requests > 1 and logging.INFO or logging.DEBUG
    logging.basicConfig(level=logging.DEBUG)
    threads = []
    for _ in range(args.num_threads):
        threads.append(Thread())
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    elapsed = (datetime.datetime.now() - start).total_seconds()
    logging.info("total processing time: %s", elapsed)
    logging.info("total number of requests: %s", args.num_requests)
    logging.info("seconds per request: %s", elapsed / args.num_requests)
    logging.info("requests per second: %s", args.num_requests / elapsed)
    open("old-val-test.xml", "wb").write(Request.responses[0])
    prev = None
    for response in Request.responses:
        this = etree.XML(response).find("CdrResponse/CdrValidateDocResp")
        this = etree.tostring(this)
        if prev and prev != this:
            raise Exception("%s doesn't match %s" % (prev, this))
        prev = this
main()
