import glob
import os
import re
from lxml import etree

try:
    os.stat("C:/Users/bkline")
    SCHEMAS = "C:/Users/bkline/Dropbox/cdr/new_server/data/schemas"
except:
    SCHEMAS = "/home/bkline/Dropbox/cdr/new_server/data/schemas"

align = "left"
builtin_types = set([
    "IDREF",
    "NMTOKEN",
    "NOTATION",
    "QName",
    "anyURI",
    "base64Binary"
    "boolean",
    "date",
    "dateTime",
    "decimal",
    "double",
    "duration",
    "float",
    "gDay",
    "gMonth",
    "gYear",
    "gYearMonth",
    "hexBinary",
    "int",
    "integer",
    "string",
    "time",
])
def unescape(me):
    node = etree.fromstring("<foo>%s</foo>" % me)
    return node.text
def fix_attributes(match):
    global align
    lines = match.group(0)[:-2].splitlines()
    stripped = lines[1].lstrip()
    padding = len(lines[1]) - len(stripped)
    width = stripped.find("=")
    required = True
    #qualified = "cdr:" in lines[0]
    quotes = '"' in lines[0]
    modified = [lines[0]] #.replace("cdr:", "")]
    for line in lines[1:]:
        name, value = line.split("=")
        if "minOccurs" in name:
            required = False
        else:
            modified.append(line)
    if required:
        if align == "left":
            line = "%s%-*s= 'required'" % (" " * padding, width, "use")
        else:
            line = "%s%*s = 'required'" % (" " * padding, width - 1, "use")
        if quotes:
            line = line.replace("'", '"')
        modified.append(line)
    if False: #qualified:
        if align == "left":
            line = "%s%-*s= 'qualified'" % (" " * padding, width, "form")
        else:
            line = "%s%*s = 'qualified'" % (" " * padding, width - 1, "form")
        if quotes:
            line = line.replace("'", '"')
        modified.append(line)
    return "\n".join(modified) + "/>"
def fix_namespaces(schema):
    global align
    lines = schema.splitlines()
    modified = []
    in_appinfo = False
    for line in lines:
        if "WHERE" not in line:
            line = line.replace("cdr:", "cdr-")
        stripped = line.strip()
        if stripped:
            if "</appInfo" in stripped:
                line = line.replace("appInfo", "xsd:appinfo")
                in_appinfo = False
            elif not in_appinfo:
                line = line.replace("</", "@@CLOSER@@")
                line = line.replace("<!", "@@ANGLEBANG@@")
                if re.match(r"<[a-zA-Z]", stripped):
                    line = line.replace("<", "<xsd:")
                    if re.match("<schema", stripped):
                        if align == "left":
                            target = "xmlns    "
                        else:
                            target = "    xmlns"
                        line = line.replace(target, "xmlns:xsd")
                    elif re.match(r"<restriction\s+base[ =]", stripped):
                        line = fix_type(line)
                    elif re.match(r"<extension\s+base[ =]", stripped):
                        line = fix_type(line)
                elif re.match(r"</[a-zA-Z]", stripped):
                    if not in_appinfo:
                        line = line.replace("</", "</xsd:")
                elif line.startswith("      "):
                    line = "    " + line
                    if not in_appinfo and "    type" in line:
                        line = fix_type(line)
        line = line.replace("@@CLOSER@@", "</xsd:")
        line = line.replace("@@ANGLEBANG@@", "<!")
        line = line.replace("xsd:appInfo", "xsd:appinfo")
        if "<xsd:appinfo" in line:
            in_appinfo = True
        if "</xsd:appinfo" in line:
            in_appinfo = False
        modified.append(line)
    return "\n".join(modified) + "\n"

def fix_type(line):
    pieces = line.split("=", 1)
    if len(pieces) != 2:
        return line
    left, right = pieces
    name = re.sub(r"[^\w]+", "", right)
    if name in builtin_types:
        new = "xsd:%s" % name
        return left + "=" + right.replace(name, new)
    return line

def fix_cardinality(schema):
    lines = schema.splitlines()
    modified = []
    for line in lines:
        if re.search(r"maxOccurs.*unlimited", line):
            line = line.replace("unlimited", "unbounded")
        elif "    minoccurs" in line:
            line = line.replace("minoccurs", "minOccurs")
        modified.append(line)
    return "\n".join(modified) + "\n"
def fix_locations(schema):
    lines = schema.splitlines()
    modified = []
    for line in lines:
        if "schemaLocation" in line:
            line = line.replace(".xml", ".xsd")
        modified.append(line)
    return "\n".join(modified) + "\n"
def fix_test(match):
    asrt = match.group(0)
    match = re.search(r"<assert\s+test\s*=([^>]+)>([^<]*)</assert>", asrt)
    if not match:
        raise Exception("no match for %s" % astr)
    test = unescape(match.group(1).strip()[1:-1])
    mesg = unescape(match.group(2).strip())
    return """\
     <assert>
      <test><![CDATA[
%s
]]>
      </test>
      <message><![CDATA[
%s
]]>
      </message>
     </assert>
""" % (test, mesg)

def fix_schema(schema):
    global align
    align = ("xmlns = \"http" in schema) and "right" or "left"
    schema = re.sub(r" *<attribute +name *= *['\"]xmlns:cdr['\"][^>]*/> *\n",
                    "", schema)
    schema = re.sub(r"<rule\s+context\s+=", "<rule context =", schema)
    schema = re.sub(" *<assert.*?</assert> *\n", fix_test, schema, flags=re.S)
    schema = re.sub(" *<attribute[^>]+/>", fix_attributes, schema)
    schema = fix_cardinality(schema)
    schema = fix_namespaces(schema)
    schema = fix_locations(schema)
    return schema
    return re.sub("<[^>]+http://www.w3.org/2001/XMLSchema[^>]*>",
                  """\
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"
            version="1.0"
            xmlns:cdr="cips.nci.nih.gov/cdr"
            targetNamespace="cips.nci.nih.gov/cdr"
            elementFormDefault="unqualified"
            attributeFormDefault="unqualified">""", schema)

for path in glob.glob("%s/original/*.xml" % SCHEMAS):
    if "schema.xml" in path:
        continue
    print(path)
    old = open(path, "rb").read()
    name = os.path.basename(path)
    new = fix_schema(str(old, "utf-8"))
    for directory in ("%s/new" % SCHEMAS, "%s/baseline" % SCHEMAS):
        path = "%s/%s.xsd" % (directory, name[:-4])
        open(path, "wb").write(bytes(new, "utf-8"))
